import os
import sys
import logging
from logging.handlers import RotatingFileHandler

from road_collisions.constants import LOG_LOCATION


FORMAT = '%(asctime)s|%(levelname)s| %(message)s'


class ColourfulFormatter(logging.Formatter):

    cyan = "\x1b[36m;21m"
    grey = "\x1b[38;21m"
    yellow = "\x1b[33;21m"
    red = "\x1b[31;21m"
    bold_red = "\x1b[31;1m"
    reset = "\x1b[0m"
    format = FORMAT
    format_problematic = FORMAT + ' (%(filename)s:%(lineno)d)'

    FORMATS = {
        logging.DEBUG: cyan + format + reset,
        logging.INFO: grey + format + reset,
        logging.WARNING: yellow + format_problematic + reset,
        logging.ERROR: red + format_problematic + reset,
        logging.CRITICAL: bold_red + format_problematic + reset
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)


logger = logging.Logger('road_collisions', logging.DEBUG)

os.makedirs(
    os.path.dirname(LOG_LOCATION),
    exist_ok=True
)

handler = RotatingFileHandler(
    LOG_LOCATION,
    maxBytes=1000000,
    backupCount=20
)
handler.setFormatter(
    logging.Formatter(
        fmt='%(asctime)s|%(levelname)s| %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S'
    )
)
logger.addHandler(handler)

std_handler = logging.StreamHandler(sys.stdout)
std_handler.setFormatter(
    ColourfulFormatter()
)
logger.addHandler(std_handler)



import copy

from unittest import TestCase

from road_collisions.utils import epsg_900913_to_4326


class UtilsTest(TestCase):

    def test_epsg_900913_to_4326(self):
        self.assertEqual(
            epsg_900913_to_4326(-697459.30722, 7042917.21888),
            (-6.265383558143537, 53.32272161460827)
        )



import os
import glob
import json

from road_collisions import logger
from road_collisions.constants import (
    WEEKDAY_MAP,
    GENDER_MAP,
    VEHICLE_TYPE_MAP,
    HOUR_MAP,
    CIRCUMSTANCS_MAP,
    SEVERITY_MAP,
    COUNTY_MAP
)
from road_collisions.models.generic import (
    GenericObject,
    GenericObjects
)
from road_collisions.models.raw_collision import RawCollision

WEEKDAY_MAP_VALS = WEEKDAY_MAP.values()
GENDER_MAP_VALS = GENDER_MAP.values()
VEHICLE_TYPE_MAP_VALS = VEHICLE_TYPE_MAP.values()
HOUR_MAP_VALS = HOUR_MAP.values()
CIRCUMSTANCS_MAP_VALS = CIRCUMSTANCS_MAP.values()
SEVERITY_MAP_VALS = SEVERITY_MAP.values()
COUNTY_MAP_VALS = COUNTY_MAP.values()


data_props = [
    'lat',
    'lng',
    'year',
    'weekday',
    'gender',
    'age',
    'vehicle_type',
    'vehicle',
    'hour',
    'circumstances',
    'num_fatal',
    'num_minor',
    'num_notinjured',
    'num_serious',
    'num_unknown',
    'speed_limit',
    'severity',
    'county',
    'carrf',
    'carri',
    'class2',
    'goodsrf',
    'goodsri',
    'mcycrf',
    'mcycri',
    'otherrf',
    'otherri',
    'pcycrf',
    'pcycri',
    'pedrf',
    'pedri',
    'psvrf',
    'psvri',
    'unknrf',
    'unknri'
]


class Collision(GenericObject, RawCollision):

    def __init__(self, **kwargs):
        self.data = {}
        for prop in data_props:
            self.data[prop] = kwargs[prop]

        super().__init__()

    @staticmethod
    def parse(data):
        if isinstance(data, Collision):
            return data

        if isinstance(data, dict):
            if 'data' in data.keys():
                return Collision(
                    **RawCollision.parse(
                        data
                    ).data
                )
            else:
                # from serialization
                return Collision(
                    **data
                )

    @property
    def id(self):
        return hash(str(self.serialize()))

    def serialize(self):
        return {
            'lat': self.geo[0],
            'lng': self.geo[1],
            'year': self.year,
            'weekday': self.weekday,
            'gender': self.gender,
            'age': self.age,
            'vehicle_type': self.vehicle_type,
            'vehicle': self.vehicle,
            'hour': self.hour,
            'circumstances': self.circumstances,
            'num_fatal': self.num_fatal,
            'num_minor': self.num_minor,
            'num_notinjured': self.num_notinjured,
            'num_serious': self.num_serious,
            'num_unknown': self.num_unknown,
            'speed_limit': self.speed_limit,
            'severity': self.severity,
            'county': self.county,
            'carrf': self.carrf,
            'carri': self.carri,
            'class2': self.class2,
            'goodsrf': self.goodsrf,
            'goodsri': self.goodsri,
            'mcycrf': self.mcycrf,
            'mcycri': self.mcycri,
            'otherrf': self.otherrf,
            'otherri': self.otherri,
            'pcycrf': self.pcycrf,
            'pcycri': self.pcycri,
            'pedrf': self.pedrf,
            'pedri': self.pedri,
            'psvrf': self.psvrf,
            'psvri': self.psvri,
            'unknrf': self.unknrf,
            'unknri': self.unknri
        }

    @property
    def geo(self):
        return [self.data['lat'], self.data['lng']]

    @property
    def lat(self):
        return self.data['lat']

    @property
    def lng(self):
        return self.data['lng']

    @property
    def year(self):
        if isinstance(self.data['year'], int):
            if self.data['year'] > 2000:
                return self.data['year']
        return int(f'20{str(self.data["year"]).zfill(2)}')

    @property
    def weekday(self):
        if self.data['weekday'] in WEEKDAY_MAP_VALS:
            return self.data['weekday']

        return WEEKDAY_MAP[
            int(self.data['weekday'])
        ]

    @property
    def gender(self):
        gender = None

        if self.data['gender'] in GENDER_MAP_VALS:
            return self.data['gender']

        try:
            gender = GENDER_MAP[
                self.data['gender'].lower()
            ]
        except KeyError:
            logger.debug('Can not parse gender: %s', self.data['gender'])
            gender = self.data['gender']

        return gender

    @property
    def age(self):
        age = None

        if isinstance(self.data['age'], int):
            if self.data['age'] % 10 == 0:
                return self.data['age']

        if self.data['age'] is None:
            return None

        try:
            age = int(self.data['age']) * 10
        except ValueError:
            logger.debug('Can not parse age: %s', self.data['age'])

        return age

    @property
    def vehicle_type(self):
        if self.data['vehicle_type'] in VEHICLE_TYPE_MAP_VALS:
            return self.data['vehicle_type']

        return VEHICLE_TYPE_MAP.get(
            int(self.data['vehicle_type']),
            self.data['vehicle_type']
        )

    @property
    def vehicle(self):
        return self.data['vehicle']

    @property
    def hour(self):
        if self.data['hour'] in HOUR_MAP_VALS:
            return self.data['hour']

        hour = None
        try:
            hour = HOUR_MAP.get(
                int(self.data['hour']),
                self.data['hour']
            )
        except ValueError:
            hour = self.data['hour']

        return hour

    @property
    def circumstances(self):
        if self.data['circumstances'] in CIRCUMSTANCS_MAP_VALS:
            return self.data['circumstances']

        circumstances = None
        try:
            circumstances = CIRCUMSTANCS_MAP.get(
                int(self.data['circumstances']),
                self.data['circumstances']
            )
        except ValueError:
            circumstances = self.data['circumstances']

        return circumstances

    @property
    def num_fatal(self):
        return int(self.data['num_fatal'])

    @property
    def num_minor(self):
        return int(self.data['num_minor'])

    @property
    def num_notinjured(self):
        return int(self.data['num_notinjured'])

    @property
    def num_serious(self):
        return int(self.data['num_serious'])

    @property
    def num_unknown(self):
        return int(self.data['num_unknown'])

    @property
    def speed_limit(self):
        if isinstance(self.data['speed_limit'], int):
            return self.data['speed_limit']

        if self.data['speed_limit'] is None:
            return None

        speed_limit = None
        try:
            speed_limit = int(self.data['speed_limit'])
        except ValueError:
            logger.debug(
                'Could not parse speed limit: %s',
                self.data['speed_limit']
            )

        return speed_limit

    @property
    def severity(self):
        if self.data['severity'] in SEVERITY_MAP_VALS:
            return self.data['severity']

        return SEVERITY_MAP[
            int(self.data['severity'])
        ]

    @property
    def county(self):
        if self.data['county'] in COUNTY_MAP_VALS:
            return self.data['county']

        return COUNTY_MAP[
            int(self.data['county'])
        ]

    # NOT SURE WHAT THE BELOW DO

    @property
    def carrf(self):
        return int(self.data['carrf'])

    @property
    def carri(self):
        return int(self.data['carri'])

    @property
    def class2(self):
        # TODO: looks interesting
        return int(self.data['class2'])

    @property
    def goodsrf(self):
        return int(self.data['goodsrf'])

    @property
    def goodsri(self):
        return int(self.data['goodsri'])

    @property
    def mcycrf(self):
        return int(self.data['mcycrf'])

    @property
    def mcycri(self):
        return int(self.data['mcycrf'])

    @property
    def otherrf(self):
        return int(self.data['otherrf'])

    @property
    def otherri(self):
        return int(self.data['otherri'])

    @property
    def pcycrf(self):
        return int(self.data['pcycrf'])

    @property
    def pcycri(self):
        return int(self.data['pcycri'])

    @property
    def pedrf(self):
        return int(self.data['pedrf'])

    @property
    def pedri(self):
        return int(self.data['pedri'])

    @property
    def psvrf(self):
        return int(self.data['psvrf'])

    @property
    def psvri(self):
        return int(self.data['psvri'])

    @property
    def unknrf(self):
        return int(self.data['unknrf'])

    @property
    def unknri(self):
        return int(self.data['unknri'])


class Collisions(GenericObjects):

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('child_class', Collision)
        super().__init__(*args, **kwargs)

    @staticmethod
    def from_file(filepath):
        data = None
        with open(filepath, 'r') as f:
            data = json.loads(f.read())

        collisions = Collisions()
        for collision_dict in data:
            collisions.append(
                Collision.parse(
                    collision_dict
                )
            )

        return collisions

    @staticmethod
    def from_dir(dirpath):
        collisions = Collisions()
        for filename in glob.iglob(f'{dirpath}/**', recursive=True):
            if os.path.splitext(filename)[-1] != '.json':
                continue
            collisions.extend(
                Collisions.from_file(
                    filename
                )._data
            )

        return collisions

    def filter(self, **kwargs):
        '''
        By whatever props that exist
        '''
        logger.debug('Filtering from %s' % (len(self)))

        filtered = [
            d for d in self if all(
                [
                    getattr(d, attr) == kwargs[attr] for attr in kwargs.keys()
                ]
            )
        ]

        return Collisions(
            data=filtered
        )

    def write(self, write_file):
        with open(write_file, 'w') as f:
            f.write(json.dumps(self.serialize()))

    @staticmethod
    def load_all():
        import road_collisions
        return Collisions.from_dir(
            os.path.join(road_collisions.__path__[0], 'resources')
        )



import os
import glob
import json

from road_collisions import logger
from road_collisions.utils import epsg_900913_to_4326
from road_collisions.models.generic import GenericObject, GenericObjects


data_props = [
    'lat',
    'lng',
    'year',
    'weekday',
    'gender',
    'age',
    'vehicle_type',
    'vehicle',
    'hour',
    'circumstances',
    'num_fatal',
    'num_minor',
    'num_notinjured',
    'num_serious',
    'num_unknown',
    'speed_limit',
    'severity',
    'county',
    'carrf',
    'carri',
    'class2',
    'goodsrf',
    'goodsri',
    'mcycrf',
    'mcycri',
    'otherrf',
    'otherri',
    'pcycrf',
    'pcycri',
    'pedrf',
    'pedri',
    'psvrf',
    'psvri',
    'unknrf',
    'unknri'
]


class RawCollision():

    def __init__(self, **kwargs):
        self.data = kwargs

    @staticmethod
    def parse(data):
        if isinstance(data, RawCollision):
            return data

        lat_lng = list(
            reversed(
                epsg_900913_to_4326(
                    data['geometry']['x'],
                    data['geometry']['y']
                )
            )
        )

        remaps = {  # These are reversed (new: old)
            'gender': 'sex',
            'num_fatal': 'no_fatal',
            'num_minor': 'no_minor',
            'num_notinjured': 'no_notinjured',
            'num_serious': 'no_serious',
            'num_unknown': 'no_unknown',
            'speed_limit': 'splimit',
            'vehicle_type': 'class1',
            'severity': 'type',
            'circumstances': 'prcoltyp'
        }

        parsed = {}

        for prop in data_props:
            parsed[prop] = data['data'].get(remaps.get(prop, prop), None)

        parsed['lat'] = lat_lng[0]
        parsed['lng'] = lat_lng[1]

        return RawCollision(
            **parsed
        )



import argparse
import json
import os
import glob

from road_collisions import logger
from road_collisions.models.collision import Collision


def main():

    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--raw-dir',
        dest='raw_dir',
        default=None
    )
    parser.add_argument(
        '--out-file',
        dest='out_file',
        default='/tmp/road_collisions_clean.json'
    )
    args = parser.parse_args()

    if args.raw_dir is None:
        raise Exception('Muse include --raw-dir')

    data = {}

    for filename in glob.iglob(f'{args.raw_dir}/**', recursive=True):
        if os.path.splitext(filename)[-1] != '.json':
            continue

        file_data = json.loads(open(filename).read())
        clean_data = file_data['controls'][3]['layers'][0]['features']

        if len(clean_data) == 0:
            continue

        for d in clean_data:
            point = Collision(d['data'], d['geometry'])
            data[point.id] = d

    list_data = data.values()
    logger.info('Resulted in %s datapoints', len(list_data))

    with open(args.out_file, 'w') as f:
        f.write(json.dumps(list(data.values())))


if __name__ == '__main__':
    main()



import uuid
from typing import (
    Any,
    Iterable,
    Mapping
)


class GenericObjects():

    def __init__(self, *args, **kwargs):
        '''

        :kwarg data:
        :kwarg child_class:
        '''
        self._uuid = kwargs.get('uuid', None)
        self.child_class = kwargs.get('child_class', GenericObject)
        self._data = [
            self.child_class.parse(d) for d in kwargs.get('data', [])
        ]

    @property
    def uuid(self):
        if self._uuid is None:
            self._uuid = uuid.uuid4()
        return self._uuid

    def __getitem__(self, i):
        return self._data[i]

    def __iter__(self):
        return (i for i in self._data)

    def __len__(self):
        return len(self._data)

    def append(self, obj: Any) -> None:
        '''

        :param obj:
        '''
        if not isinstance(obj, self.child_class):
            obj = self.child_class.parse(obj)
        self._data.append(obj)

    def extend(self, objs: Iterable[Any]) -> None:
        '''

        :param objs:
        '''
        for obj in objs:
            self.append(obj)

    def serialize(self) -> Iterable[Mapping[str, Any]]:
        return [obj.serialize() for obj in self.data]

    def reload(self) -> None:
        '''
        Reload to get rid of object cached properties. This should be done on
        any modifications to the data within any of the object's objects.
        '''
        self._data = [
            self.child_class(data=obj.data) for obj in self.data if len(obj.data) > 0
        ]

    def remove(self, obj: Any) -> None:
        if not isinstance(obj, self.child_class):
            raise TypeError('Bad type: %s' % (type(obj)))
        self._data = [
            o for o in self._data if o.id != obj.id
        ]

    @staticmethod
    def parse(objs):
        raise NotImplementedError()

    @property
    def data(self) -> Iterable[Any]:
        return self._data


class GenericObject():

    def __init__(self, *args, **kwargs):
        '''

        :kwarg data:
        '''
        self._uuid = kwargs.get('uuid', None)

    @property
    def uuid(self):
        if self._uuid is None:
            self._uuid = uuid.uuid4()
        return self._uuid

    def __repr__(self):
        return str('%s(%s)' % (
            type(self).__name__,
            self.uuid
        ))

    @staticmethod
    def parse(obj):
        raise NotImplementedError()

    def serialize(self):
        raise NotImplementedError()



from road_collisions import logger
from road_collisions.models.collision import Collisions


def main():

    collisions = Collisions.load_all()

    logger.info('Loaded %s collisions', (len(collisions)))
    logger.info('Do something with the data in the variable \'collisions\'...')

    import pdb; pdb.set_trace()

    pass



if __name__ == '__main__':
    main()



from setuptools import (
    find_packages,
    setup
)

INSTALL_REQUIRES = (
)

setup(
    name='road_collisions',
    version='0.0.2',
    python_requires='>=3.6',
    description='Road collision data and tooling',
    author='Robert Lucey',
    url='https://github.com/RobertLucey/road-collisions',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    install_requires=INSTALL_REQUIRES,
    package_data={
        'road_collisions': ['resources/ireland.json']
    },
    entry_points={
        'console_scripts': [
            'raw_cleaner = road_collisions.bin.raw_cleaner:main',
            'load_road_collisions = road_collisions.bin.load:main',
        ]
    }
)



LOG_LOCATION = '/opt/log/road_collisions/road_collisions.log'


WEEKDAY_MAP = {
    1: 'sunday',
    2: 'monday',
    3: 'tuesday',
    4: 'wednesday',
    5: 'thursday',
    6: 'friday',
    7: 'saturday'
}

GENDER_MAP = {
    'm': 'male',
    'f': 'female'
}

VEHICLE_TYPE_MAP = {
    1: 'bicycle',
    2: 'motorcycle',
    3: 'car',
    4: 'goods_vehicle',
    5: 'bus',
    6: 'other'
}

HOUR_MAP = {
    1: '7-10',
    2: '10-16',
    3: '16-19',
    4: '19-23',
    5: '23-3',
    6: '3-7'
}

CIRCUMSTANCS_MAP = {
    1: 'pedestrian',
    2: 'single_vehicle_only',
    3: 'head_on_conflict',
    4: 'head_on_right_turn',
    5: 'angle_both_straight',
    6: 'angle_right_turn',
    7: 'rear_end_straight',
    8: 'rear_end_right_turn',
    9: 'rear_end_left_turn',
    10: 'other'
}

SEVERITY_MAP = {
    0: 'all',
    1: 'fatal',
    2: 'serious',
    3: 'minor'
}

COUNTY_MAP = {
    1: 'carlow',
    2: 'cavan',
    3: 'clare',
    4: 'cork',
    5: 'donegal',
    6: 'dublin',
    7: 'galway',
    8: 'kerry',
    9: 'kildare',
    10: 'kilkenny',
    11: 'laois',
    12: 'leitrim',
    13: 'limerick',
    14: 'longford',
    15: 'louth',
    16: 'mayo',
    17: 'meath',
    18: 'monaghan',
    19: 'offaly',
    20: 'roscommon',
    21: 'sligo',
    22: 'Tipperary',
    23: 'Waterford',
    24: 'Westmeath',
    25: 'wexford',
    26: 'wicklow'
}
